﻿using Abp.AspNetCore;
using Abp.AspNetCore.Configuration;
using Abp.Modules;
using Abp.Reflection.Extensions;
using KDTEC.ProManager.Configuration;
using KDTEC.ProManager.EntityFrameworkCore;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc.ApplicationParts;
using Microsoft.Extensions.Configuration;

namespace KDTEC.ProManager.Web.Startup
{
    [DependsOn(
        typeof(ProManagerApplicationModule), 
        typeof(ProManagerEntityFrameworkCoreModule), 
        typeof(AbpAspNetCoreModule))]
    public class ProManagerWebModule : AbpModule
    {
        private readonly IConfigurationRoot _appConfiguration;

        public ProManagerWebModule(IHostingEnvironment env)
        {
            _appConfiguration = AppConfigurations.Get(env.ContentRootPath, env.EnvironmentName);
        }

        public override void PreInitialize()
        {
            Configuration.DefaultNameOrConnectionString = _appConfiguration.GetConnectionString(ProManagerConsts.ConnectionStringName);

            Configuration.Navigation.Providers.Add<ProManagerNavigationProvider>();

            Configuration.Modules.AbpAspNetCore()
                .CreateControllersForAppServices(
                    typeof(ProManagerApplicationModule).GetAssembly()
                );
        }

        public override void Initialize()
        {
            IocManager.RegisterAssemblyByConvention(typeof(ProManagerWebModule).GetAssembly());
        }

        public override void PostInitialize()
        {
            IocManager.Resolve<ApplicationPartManager>()
                .AddApplicationPartsIfNotAddedBefore(typeof(ProManagerWebModule).Assembly);
        }
    }
}